""" Graph package """
import types
import Set
import Multimap

class GraphException(Exception):
	def __init__(self,args=()):
		Exception.__init__(self,args)
class EdgeException(GraphException):
	def __init__(self,args=()):
		GraphException.__init__(self,args)
class NodeException(GraphException):
	def __init__(self,args=()):
		GraphException.__init__(self,args)

class Graph:
	"""Graph(), Graph(es)
	creates a directed graph initialized to contain all the
		edges contained as pairs in es
	Edges are unordered pairs of nodes. They are placed in a 
		canonical order.
	Nodes are objects that can be used as 
		keys of dictionaries and can be compared with <=.
	"""
	def __init__(self,es=()):
		self.nodeset=Set.Set()
		self.adjacentNodes=Multimap.mmap()
		for e in es:
			self.addEdge(e)

	def nodes(self):
		"""nodes(self)
		returns a pure set of all the nodes in self
		"""
		return self.nodeset.copy()
	def edges(self): 
		"""edges(self)
		returns a list of all the edges in self
		"""
		s=Set.Set()
		for n in self.nodeset:
		    for m in self.adjacentNodes[n]:
			if n <= m:
				s.insert((n,m))
		return s
	def addNode(self,n):
		"""addNode(self,n)
		adds the node n to self
		"""
		self.nodeset.insert(n)
		return n
	def deleteNode(self,n):
		"""deleteNode(self,n)
		deletes the node n and all incident edges from self
		"""
		if not self.nodeset.contains(n):
			return None
		for m in self.adjacentNodes[n].items():
			self.adjacentNodes.delete(m,n)
		del self.adjacentNodes[n]
		self.nodeset.delete(n)
		return n
	def addEdge(self,*ns):
		"""addEdge(self,x[,y])
		adds the edge x or (x,y) to self
		"""
		n0,n1=self.__getArgEdge(ns)
		self.addNode(n0)
		self.addNode(n1)
		self.adjacentNodes[n0]=n1
		self.adjacentNodes[n1]=n0
		return self.__constructEdge(n0,n1)
	def deleteEdge(self,*ns):
		"""deleteEdge(self,x[,y])
		deletes the edge x or (x,y) from self
		"""
		n0,n1=self.__getArgEdge(ns)
		if not self.nodeset.contains(n0) or  \
		    not self.nodeset.contains(n1):
			return None
		if not self.adjacentNodes[n0].contains(n1):
			return None
		self.adjacentNodes.delete(n0,n1)
		self.adjacentNodes.delete(n1,n0)
		return self.__constructEdge(n0,n1)
	def containsNode(self,n):
		"""containsNode(self,n)
		returns true if self contains node n
		"""
		return self.nodeset.contains(n)
	def containsEdge(self,*ns):
		"""containsEdge(self,x[,y])
		returns true if self contains edge x or (x,y)
		"""
		n0,n1=self.__getArgEdge(ns)
		if self.nodeset.contains(n0):
		    return self.adjacentNodes[n0].contains(n1)
		else: return 0
	def __getArgEdge(self,ns):
		if len(ns)==2:
			n0=ns[0]
			n1=ns[1]
		elif len(ns)==1 and \
		    type(ns[0])==types.TupleType and \
		    len(ns[0])==2:
			e=ns[0]
			n0=e[0]
			n1=e[1]
		else: raise EdgeException(repr(ns))
		return n0,n1
	def degree(self,n): 
		"""degree(self,n)
		returns the number of edges incident to node n
		A self-loop is counted as 1.
		"""
		if not self.nodeset.contains(n):
			raise NodeException(n)
		return len(self.adjacentNodes[n])
	def incident(self,n):
		"""incident(self,n)
		returns a list of all edges incident to node n
		"""
		if not self.nodeset.contains(n):
			raise NodeException(repr(n))
		s=Set.Set()
		for m in self.adjacentNodes[n]:
			s.insert(self.__constructEdge(m,n))
		return s
	def adjacent(self,n):
		"""adjacent(self,n)
		returns a set of all x s.t. (n,x) is an 
		edge in self
		"""
		if not self.nodeset.contains(n):
			raise NodeException(repr(n))
		return PureSet.PureSet(rep=self.adjacentNodes[n])
	def copy(self):
		"""copy(self)
		creates a copy of graph self
		"""
		g=Graph()
		g.nodeset=self.nodeset.copy()
		g.adjacentNodes=Multimap.mmap()
		for n in self.adjacentNodes.keys():
		    for m in self.adjacentNodes[n]:
			g.adjacentNodes[n]=m
		return g
	def new(self):
		"""new(self)
		creates a new digraph
		"""
		return Graph()
	def __constructEdge(self,n0,n1):
		if n0<=n1:
			return (n0,n1)
		else:	return (n1,n0)
	def __repr__(self):
		"""__repr__(self), repr(self)
		returns an expression that will create a copy of
		digraph self if all nodes have a repr value that
		can reconstruct them.
		"""
		return "Graph.Graph("+repr(self.edges().members())+")"
	def __str__(self):
		"""__str__(self), str(self)
		returns a string representation of the digraph self
		"""
		return repr(self)



